<?php

namespace Drupal\pocket\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\KeyValueStore\KeyValueStoreInterface;
use Drupal\Core\Url;
use Drupal\pocket\Client\PocketAuthClient;
use Drupal\pocket\Client\PocketClientFactoryInterface;
use Drupal\pocket\Exception\PocketHttpException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use function assert;

/**
 * Internal Controller that handles the authorization step of OAuth.
 *
 * PocketAuthClient::authorize() will store the callback and tell Pocket to
 * redirect the user to this controller (PocketAuthorize::authorize()) once the
 * application is authorized.
 *
 * This controller then loads the stored callback and runs it.
 *
 * @internal
 */
class PocketAuthorize extends ControllerBase {

  use DependencySerializationTrait;

  /**
   * The key-value store.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueStoreInterface
   */
  protected $storage;

  /**
   * The Pocket client factory service.
   *
   * @var \Drupal\pocket\Client\PocketClientFactoryInterface
   */
  protected $clientFactory;

  /**
   * The Pocket auth client.
   *
   * @var \Drupal\pocket\Client\PocketAuthClient
   */
  private $client;

  /**
   * PocketAuthorize constructor.
   *
   * @param \Drupal\Core\KeyValueStore\KeyValueStoreInterface $storage
   *   The key-value store.
   * @param \Drupal\pocket\Client\PocketClientFactoryInterface $clientFactory
   *   The Pocket client factory service.
   */
  public function __construct(KeyValueStoreInterface $storage, PocketClientFactoryInterface $clientFactory) {
    $this->storage = $storage;
    $this->clientFactory = $clientFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var \Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface $kve */
    $kve = $container->get('keyvalue.expirable');
    return new static(
      $kve->get('pocket'),
      $container->get('pocket.client')
    );
  }

  /**
   * Accept confirmation of a request token, and run the callback.
   *
   * @param string $id
   *   The client-side ID previously generated by PocketAuthClient::authorize().
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to the URL returned from the callback passed to
   *   PocketAuthClient::authorize().
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   If the request identifier does not exist, or Pocket does not return
   *   an access token.
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function authorize(string $id): RedirectResponse {
    $request = $this->storage->get("request:$id");
    if (!$request) {
      throw new NotFoundHttpException('This request does not exist.');
    }

    $callback = $request['callback'];
    try {
      $access = $this->getClient()
        ->getAccessToken($request['token'])
        ->setState($request['state']);
    }
    catch (PocketHttpException $exception) {
      throw new NotFoundHttpException('Pocket did not return an access token.');
    }

    $url = $callback($access);
    assert($url instanceof Url);

    return new RedirectResponse($url->toString());
  }

  /**
   * Initialize the client.
   *
   * @return \Drupal\pocket\Client\PocketAuthClient
   *   The Pocket auth client.
   */
  protected function getClient(): PocketAuthClient {
    if ($this->client === NULL) {
      $this->client = $this->clientFactory->getAuthClient();
    }
    return $this->client;
  }

}
